

#if !defined TREE_LIB_H
	#define TREE_LIB_H
	#include "../Header/TreeLib.h" 
	#include "../Header/LowerBoundUngherese.h"
#endif

#include "string.h"
#include <stdlib.h>
#include <stdio.h>

#define ROOT_LEVEL 1
#define NOT_VISITED -1
#define MAX_PROBLEM 1

void Init_BrenchStruct(int N, TBrenchStruct **tree,TStat * stat)
{
 (*tree) = ( TBrenchStruct *)malloc(sizeof( TBrenchStruct));
 (*tree)->SequenceValue = (int*) malloc(N*sizeof(int));
 (*tree)->Level=ROOT_LEVEL;
 (*tree)->PartialValue=0.0;
 (*tree)->NotVisitedNodes=(int *)malloc ( (N-1)*sizeof(int));
 (*tree )->NotVisitedNodesPerLowerBound=(int *)malloc ( (N)*sizeof(int));

 for ( int i=0; i<N; ++i )
 {
		(*tree)->SequenceValue[i]= NOT_VISITED;
		
		if(i!=0)
			(*tree )->NotVisitedNodes[i-1]=i;

		(*tree )->NotVisitedNodesPerLowerBound[i]=i;
 }

 (*tree)->SequenceValue[0]=0;
 (*tree)->Next=NULL;
 (*tree)->LowerBound=0; 



 }



void RootExtract( TBrenchStruct ** tree,TBrenchStruct ** padre)
{
	(*padre) = (*tree);
	(*tree) = (*padre)->Next;
	(*padre)->Next = NULL;
}




void DeleteNode(TBrenchStruct ** node)
{
	free((*node)->SequenceValue);
    free((*node)->NotVisitedNodes);
	free((*node)->NotVisitedNodesPerLowerBound);
    free((*node));
}
//Alloca la strittura per contenere i dati del figlio
TBrenchStruct * AllocSon(int N,TBrenchStruct*padre,int node)
{
    TBrenchStruct * p;
	int tmp;
    p= (TBrenchStruct *) malloc (sizeof(TBrenchStruct));
	p->PartialValue=0;
	p->Next=NULL;
	p->SequenceValue= (int*)malloc(N*sizeof(int));
    p->NotVisitedNodes=(int *)malloc ( (N-padre->Level)*sizeof(int));
	p->NotVisitedNodesPerLowerBound=(int *)malloc ( (N-padre->Level)*sizeof(int));
	tmp=padre->NotVisitedNodesPerLowerBound[N-padre->Level];


	//Copio all'ultima posizione il nodo che ho visitati nel padre
    tmp=padre->NotVisitedNodes[N-padre->Level-1];


	//Copio i nodi non visitati dal padre al figlio omettendo l'ultima casella dell'array
    //perchè il padre è il pen'ultimo ed è già stato visitato.
    memcpy(p->NotVisitedNodes,padre->NotVisitedNodes,sizeof(int)*(N-padre->Level));
    memcpy(p->NotVisitedNodesPerLowerBound,padre->NotVisitedNodesPerLowerBound,sizeof(int)*(N-padre->Level));


	//Tramando l'array Sequence value da padre a figlio 
    memcpy(p->SequenceValue, padre->SequenceValue, N*sizeof(int));

	//Aggiungo alla sequenza dei nodi visitati i nodo corrente
    p->SequenceValue[padre->Level]=p->NotVisitedNodes[node];    

	p->NotVisitedNodes[node]=tmp;
    p->NotVisitedNodesPerLowerBound[node]=tmp;


	//Incremento il livello
    p->Level=padre->Level+ 1;

	p->LowerBound=0.0;

  return(p);
}


void Search(TBrenchStruct ** tree,TStat stat,double ** DistanceMatrix,int N,TBrenchStruct** best, long *nodicreati, double * ub,TBrenchStruct** last,int *MaxLung)
{
    TBrenchStruct *padre=NULL;
    TBrenchStruct *tmp=NULL;
	int i=0;		

  while ((*tree) != NULL)
  {		
		RootExtract(&(*tree),&padre);
		MakeSon(&(*tree),padre,N,DistanceMatrix,&(*best),&(*nodicreati),&(*ub),&(*last),&(*MaxLung));
		DeleteNode(&padre);
	
  }
}


void MakeSon(TBrenchStruct ** tree,TBrenchStruct *padre, int N, double ** DistanceMatrix, TBrenchStruct **best,long *nodicreati, double * ub,TBrenchStruct** last,int *MaxLung)
{
    TBrenchStruct * figlio;
    int i=0;
	int tmp_lung=0;
	double UbTmp;
	//double ub=UB;


	//Ciclo su tutti i nodi non visitati
	for(i=0;i<(N-padre->Level);i++)
    {
		 //Alloco spazio in memoria per il figlio
		 figlio = AllocSon(N,padre,i);

		 //Aggiorno il valore della soluzione	
		 figlio->PartialValue = padre->PartialValue + DistanceMatrix[padre->SequenceValue[padre->Level-1]][figlio->SequenceValue[figlio->Level-1]];
		 
		 //Calcolo un lower Bound(nel )
		 UbTmp=figlio->PartialValue+UpperBound(DistanceMatrix,N,(N-figlio->Level),figlio->NotVisitedNodes,figlio->SequenceValue[figlio->Level-1]);
		 if((*ub)>UbTmp)
			(*ub)=UbTmp;



		(++(*nodicreati));
		
		
		
		 //Se sono al livello di foglia
         if(figlio->Level == N)
         {  
			figlio->PartialValue = figlio->PartialValue + DistanceMatrix[figlio->SequenceValue[figlio->Level-1]][ROOT_LEVEL];
			Elabora_foglia((*tree),&figlio,&(*best),&(*ub));
         }
         else
         {
			 //Calcola il lower bound e Upper Bound per il figlio
			 if(figlio->Level==N-1)
			 {
				 //Sono al livello foglia il lower bound sarà la distanza tra il primo e unico dei nodi non visitati e l'ultimo della sequenza
				 figlio->LowerBound=figlio->PartialValue + DistanceMatrix[figlio->SequenceValue[figlio->Level-1]][figlio->NotVisitedNodes[0]] ;
			} 
			 else
			 {
			//	 Sol_disp(figlio,N);
				
				figlio->LowerBound=figlio->PartialValue +LowerBound(DistanceMatrix,N,(N-padre->Level),figlio->NotVisitedNodesPerLowerBound) ;
			 }
		
			
			if(figlio->LowerBound<(*ub))
			{
				//inserisce il figlio nella lista dei nodi aperti
				
				Insert_best_first(&figlio,&(*tree));
				tmp_lung=Lunghezza_lista(*tree);
				if(tmp_lung >(*MaxLung))
					(*MaxLung)=tmp_lung  ;
			}
			else
			{
				DeleteNode(&figlio);
				
			}

         }//End if
       }//End for
 
    }

  




void Elabora_foglia(TBrenchStruct *tree,TBrenchStruct ** figlio,TBrenchStruct **best,double * ub)
{
  if ((*best)== NULL)
  {
    (*best) = (*figlio);  
	(*figlio)=NULL;
	
  }
    else if((*best)->PartialValue > (*figlio)->PartialValue)
    {
       DeleteNode(&(*best));
       *best = *figlio;
       *figlio=NULL;
	   
    }
    else DeleteNode(&(*figlio));

	if(*ub >(*best)->PartialValue)
		*ub=(*best)->PartialValue;
   
 
}

void  Insert_best_first(TBrenchStruct **nodo,TBrenchStruct **tree)
//inserimento ordinato in ordine decrescente rispetto al lower bound 
{
	
	
	//Rappresenta il nodo sucessivo
	TBrenchStruct* temp_pointer_succ=NULL;

	//Rappresenta i nodo precedente
	TBrenchStruct* temp_pointer_prec = (*tree);
	
	//Lista vuota
	if(temp_pointer_prec==NULL)
	{
		//inserimento i testa
		 (*nodo)->Next = (*tree);
		 (*tree) = (*nodo);  
		return;
	}

	
	//Controllo che il precedente non punti NULL
	//in modo da prendere il successivo 
	if (temp_pointer_prec != NULL)
	{
		//Pendo il successivo
		temp_pointer_succ =  temp_pointer_prec->Next;
	}
 

	
	//controllo se il nodo sia da inserire in testa
	if((*nodo)->LowerBound >= temp_pointer_prec->LowerBound)
	{
		//inserimento i testa
		(*nodo)->Next = (*tree);
		(*tree) = (*nodo); 
		return;
	}
	else
	{	
		//Se esiste solo un nodo allora
		if(temp_pointer_succ==NULL)
		{
			//Inserimeneto in coda
			temp_pointer_prec->Next=(*nodo);
			return;
		}
	}
	
	
	
	//Ciclo per l'inserimento del nodo nella lista
	while(true)
	{
		//Se il nodo da inserire è minore del precedente e  maggiore o uguale del nodo da inserire
		if((*nodo)->LowerBound < temp_pointer_prec->LowerBound && (*nodo)->LowerBound >= temp_pointer_succ->LowerBound)
		{
			temp_pointer_prec->Next=(*nodo);
			(*nodo)->Next=temp_pointer_succ;
			return;
		}
		
		//Pendo il successivo
		temp_pointer_prec=temp_pointer_prec->Next;
		
		//Se è possibile continuo a scorere la lista 
		if(temp_pointer_prec->Next!=NULL)
		{
			temp_pointer_succ =temp_pointer_prec->Next;
		}
		//Altrimenti inserisco il nodo alla fine 
		else
		{
			temp_pointer_prec->Next=(*nodo);
			return;
		}
	}

  
}

void Insert_deep_first(TBrenchStruct **nodo,TBrenchStruct **tree)
{
	(*nodo)->Next = (*tree);
	(*tree) = (*nodo);  // inserimento in testa
}


void Insert_breath_first(TBrenchStruct **nodo,TBrenchStruct **tree,TBrenchStruct **last)
{
	TBrenchStruct* tmp = (*tree);
	if(Lunghezza_lista (*tree)==0)
	{
		(*nodo)->Next = (*tree);
		(*tree) = (*nodo);
		(*last)=(*nodo);
	}
	else
	{
		(*last)->Next=(*nodo);
		(*last)=(*nodo);

	}
}




void Sol_disp(TBrenchStruct* p,int N)
{

  int j=0;
  printf ("\nLa sequenza dei nodi e':\n");
  for(j=0;j<(p->Level);j++)  
	  printf(  "%d ", p->SequenceValue[j]) ;

  printf("\nIl livello e' %d \n", p->Level) ;
  printf("Il costo parziale della soluzione e' %f \n", p->PartialValue) ;
  printf("I nodi non visitati per lower bound sono :\n") ;
  
for(j=0;j<N-(p->Level)+1;j++)
	  printf( "%d ", p->NotVisitedNodesPerLowerBound[j]) ;

printf("I nodi non visitati sono :\n") ;
  for(j=0;j<N-(p->Level);j++)
	  printf( "%d ", p->NotVisitedNodes[j]) ;


}



unsigned int Lunghezza_lista (TBrenchStruct* tree)
{
  TBrenchStruct* tmp = tree;
  unsigned int counter = 0;
  while (tmp != NULL)
  {
    ++counter;
    tmp = tmp->Next; 

  }
  return counter;
}

void PrintList(TBrenchStruct* tree)
{
	TBrenchStruct* tmp = tree;
	printf("\n");
	while (tmp != NULL)
	{
		printf("%f\t",tmp->LowerBound);
		tmp = tmp->Next; 
	}

}